pub fn init(kaddr_space: AddressSpace)Expand description
Initializes the scheduler. This should be called once during kernel initialization, before any threads are created.
kaddr_space - The address space of the kernel task. This is used to create the kernel task, which is required for the scheduler to function.
If the kernel task cannot be created, this function will panic. Note that the kernel task is essential for the system to function, so we cannot continue without it.