#[unsafe(no_mangle)]pub extern "C" fn sched_enter(ctx: *mut c_void) -> *mut c_voidExpand description
This will be called by the architecture-specific code to enter the scheduler. It will land the current thread, pick the next thread to run, and return its context and task.